@@ -3,7 +3,7 @@
 
   Sybase integration to QORE
 
-  Copyright (C) 2007 Qore Technologies
+  Copyright (C) 2022 Qore Technologies
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,7 @@
 #define _QORE_SYBASE_MODULE_H
 
 #include "../config.h"
-#include <qore/Qore.h>
+#include "qore/Qore.h"
 
 extern QoreStringNode *sybase_module_init();
 extern void sybase_module_ns_init(QoreNamespace *rns, QoreNamespace *qns);
